"""Settings loaded from the environment."""

import os
import sys

sys.argv[0] = 'sappy'  # use a consistent name regardless of program invocation

DEFAULT_PATHS = 'dist', 'site', '_site', 'public'

HOST = os.getenv('SAPPY_HOST', "")
PORT = int(os.getenv('SAPPY_PORT', 8080))


def get_address(port=None):
    """Get the (host, port) for server bindings."""
    return HOST, port or PORT



"""The HTTP server."""

import os
from pathlib import Path
from http.server import HTTPServer

from . import __version__, settings
from .handlers import SinglePageApplicationHandler


def init(root, port):
    """Create a new HTTP daemon to run."""
    path = Path.cwd()
    for root in [root, *settings.DEFAULT_PATHS]:
        if root and Path(root).exists():
            path = Path(root).resolve()  # pylint: disable=redefined-variable-type
            os.chdir(str(path))
            break

    address = settings.get_address(port=port)
    httpd = HTTPServer(address, SinglePageApplicationHandler)

    return path, httpd



# pylint: disable=missing-docstring,unused-variable,expression-not-assigned

from importlib import reload

from expecter import expect

from sappy import settings


def describe_get_address():

    def it_loads_host_from_env(monkeypatch):
        monkeypatch.setenv('SAPPY_HOST', "foobar")
        reload(settings)

        expect(settings.get_address()) == ("foobar", 8080)

    def it_loads_port_from_env(monkeypatch):
        monkeypatch.setenv('SAPPY_PORT', "1234")
        reload(settings)

        expect(settings.get_address()) == ("", 1234)



# pylint: disable=missing-docstring,unused-variable,redefined-outer-name,unused-argument,expression-not-assigned

import os
import sys
import subprocess
import time

import pytest
from expecter import expect
from click.testing import CliRunner
import requests

from sappy.cli import main


def cli(*options):
    args = sys.executable, '-m', 'sappy', *options
    process = subprocess.Popen(args)
    time.sleep(0.5)
    return process


@pytest.fixture
def temp(tmpdir):
    tmpdir.chdir()


@pytest.fixture
def temp_with_dist(temp):
    os.mkdir("dist")


def describe_cli():

    def with_defaults(temp_with_dist):
        process = cli()

        try:
            response = requests.get("http://localhost:8080/foobar")
        finally:
            process.kill()

        expect(response.status_code) == 200

    def with_custom_directory(temp):
        process = cli('.')

        try:
            response = requests.get("http://localhost:8080/foobar")
        finally:
            process.kill()

        expect(response.status_code) == 200

    def with_custom_port(temp_with_dist):
        process = cli('--port=1234')

        try:
            response = requests.get("http://localhost:1234/foobar")
        finally:
            process.kill()

        expect(response.status_code) == 200

    def with_an_invalid_directory(temp):
        runner = CliRunner()
        result = runner.invoke(main, ['unknown/directory'])

        expect(result.exit_code) == 2



"""Request handlers."""

from http.server import SimpleHTTPRequestHandler
from urllib.parse import urlparse
from pathlib import Path


class SinglePageApplicationHandler(SimpleHTTPRequestHandler):
    """Handler to pass all requests to the index."""

    def do_GET(self):
        """Override GETs for unknown paths."""
        params = urlparse(self.path)
        path = Path(params.path.strip('/'))

        if path.exists():
            super().do_GET()

        else:
            self.send_response(200)
            self.send_header('Content-Type', 'text/html')
            self.end_headers()
            with open("index.html", 'rb') as rfile:
                self.copyfile(rfile, self.wfile)



"""Command-line interface."""

import click

from . import __version__
from . import server

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

ReadableDirectory = click.Path(exists=True, readable=True, file_okay=False)


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument('root', type=ReadableDirectory, required=False)
@click.option('-p', '--port', type=int, help="Port for the web server.")
@click.option('-l', '--launch', is_flag=True, help="Launch the page's index.")
@click.version_option(version=__version__)
def main(root=None, port=None, launch=False):
    """Serve a single-page application from a directory."""
    path, httpd = server.init(root, port)

    click.echo("Serving {d}/ on {p}".format(d=path, p=httpd.server_port))
    if launch:
        click.launch("http://localhost:{p}".format(p=httpd.server_port))
    httpd.serve_forever()
